/*
 * ImageCanvas.java
 *
 * Created on February 25, 2002, 1:29 AM
 */
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.net.URL;
import java.net.MalformedURLException;
/**
 *
 * @author  mike
 */
public class ImageCanvas extends Canvas {
    private Dimension preferredSize;
    private Image currentImage;
    private PopupMenu popup;
    private ImageDownloader downloader;
    private String host;
    private int port;
    private ImageCanvas thisImageCanvas;
    private double fps=0.0;
    private Applet applet;
    private Image offImage;
    private Graphics offGraphics;
    private MenuItem startStop;
    private Frame about;

    /** Creates a new instance of ImageCanvas */
    public ImageCanvas(Applet a, String host, int port, int width,int height) {
        applet = a;
        this.host = host;
        this.port = port;
        preferredSize = new Dimension(width,height);
        if (a == null){
            setImage(Toolkit.getDefaultToolkit().getImage("init.jpg"));
        }else{
            try{
                setImage(a.getImage(new URL(a.getCodeBase().toString() + "init.jpg")));
            }catch(MalformedURLException e){}
        }
        about = createAbout();
        add(createMenu());
        thisImageCanvas = this;
        
        // mouse listener
        addMouseListener(createMouseListener());
        downloader = new ImageDownloader(host,port,this);
        downloader.start();
    }
    
    public Dimension getPreferredSize(){
        return preferredSize;
    }
    /**
     * returns the time it took to set the image 
     */
    public int setImage(Image img){
        //if (currentImage  != null) currentImage.flush();
        int count = 0,waitTime=5;
        while (!prepareImage(img,this)){
            wait(waitTime);
            count ++;
            if (count * waitTime > 5000) break;
        }
        currentImage = img;
        repaint();
	return (count * waitTime);
    }
    
    
    public void paint(Graphics g){
	update(g);
    }
    
    public void update(Graphics g){
        if (currentImage != null){
	    //dubble buffer
            offImage = createImage(preferredSize.width,preferredSize.height);
            offGraphics = offImage.getGraphics();
            offGraphics.drawImage(currentImage,0,0,preferredSize.width,preferredSize.height,this);
            g.drawImage(offImage,0,0,this);
            
            //no double buffer
           // g.drawImage(currentImage,0,0,preferredSize.width,preferredSize.height,this);
        }
    }

    public Image getImage(){
        return null;
    }
    
    public void wait(int n){
        try{
           Thread.sleep(n);
        }catch(InterruptedException e){}
    }
    
    public int getPort(){
        return port;
    }
    public String getHost(){
        return host;
    }
    public void disconnect(){
         downloader.disconnect();
    }
    public boolean isDoubleBuffered(){
        return true;
    }

    public MouseListener createMouseListener(){
        return (new MouseListener(){
            public void mouseClicked(MouseEvent e){
                //System.out.println("Clicked: " + e.toString());
            }
            public void mouseEntered(MouseEvent e){
                //System.out.println("Entered: " + e.toString());
            }
            public void mouseExited(MouseEvent e){
                //System.out.println("Exited: " + e.toString());
              
            }
            public void mousePressed(MouseEvent e){
                //System.out.println("Pressed: " + e.toString());
            }
            public void mouseReleased(MouseEvent e){
                //System.out.println("Released: " + e.toString());
                switch(e.getModifiers()){
                    case 16:
                        //left
                        //System.out.println("left");
                        break;
                    case 8:
                        //middle
                        //System.out.println("middle");
                        break;
                    case 4:
                        if (downloader == null || !downloader.isConnected()){
                            thisImageCanvas.startStop.setLabel("Start");
                        }else{
                            thisImageCanvas.startStop.setLabel("Stop");
                        }
                        popup.show(e.getComponent(),e.getX(), e.getY());
                        

                        //System.out.println("right");
                        break;
                    default:
                        break;                       
                }                  
            }
        });
    }

    public Frame createAbout(){
        Frame f = new Frame("About");
        Panel p = new Panel();
	  p.setLayout(new GridLayout(0,1));
        
        p.add(new Label(" "));
        p.add(new Label("Copyright (C) 2002"));
        p.add(new Label("Mike Morrison"));
        p.add(new Label(" "));
 
        Button b = new Button("Ok");
        b.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                about.hide();
            }
        });
        f.add(p,BorderLayout.NORTH);
        p = new Panel();
        p.setLayout(new FlowLayout());
        p.add(b);
        f.add(p,BorderLayout.CENTER);
        f.pack();
        f.setBackground(Color.lightGray);
        f.setResizable(false);
        return f;          
    }
    public PopupMenu createMenu(){

        //create the menu
        MenuItem menuItem;
        popup = new PopupMenu("WebCam Options");
        startStop = new MenuItem("Stop");
        startStop.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                if (downloader == null || !downloader.isConnected()){
                    downloader = new ImageDownloader(thisImageCanvas.getHost(),thisImageCanvas.getPort(),thisImageCanvas);
                    downloader.start();
                    downloader.setMaxFPS(thisImageCanvas.fps);
                }else{
                    thisImageCanvas.fps = downloader.getMaxFPS();
                    downloader.disconnect();
                    downloader = null;
                }
                
            }
        });
        popup.add(startStop);
        
        
        //frames per second submenu
        Menu subMenu = new Menu("Max FPS");
        menuItem = new MenuItem("0.1");
        menuItem.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                //set maxfps to 0.1
                if (downloader != null){
                    downloader.setMaxFPS(.1);
                    thisImageCanvas.fps = .1;
                }
            }
        });
        subMenu.add(menuItem);

        menuItem = new MenuItem("0.2");
        menuItem.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                //set maxfps to 0.2
                if (downloader != null){
                    downloader.setMaxFPS(.2);
                    thisImageCanvas.fps = .2;
                }                
            }
        });
        subMenu.add(menuItem);
        menuItem = new MenuItem("0.5");
        menuItem.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                //set maxfps to 0.5
                if (downloader != null){
                    downloader.setMaxFPS(.5);
                    thisImageCanvas.fps =.5;
                }
            }
        });
        subMenu.add(menuItem);
        menuItem = new MenuItem("1");
        menuItem.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                //set maxfps to 1
                if (downloader != null){
                    downloader.setMaxFPS(1.0);
                    thisImageCanvas.fps = 1.0;
                }
            }
        });
        subMenu.add(menuItem);
        menuItem = new MenuItem("2");
        menuItem.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                //set maxfps to 2
                if (downloader != null){
                    downloader.setMaxFPS(2.0);
                    thisImageCanvas.fps = 2.0;
                }
            }
        });
        subMenu.add(menuItem);
        menuItem = new MenuItem("5");
        menuItem.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                //set maxfps to 5
                if (downloader != null){
                    downloader.setMaxFPS(5.0);
                    thisImageCanvas.fps = 5.0;
                }
            }
        });
        subMenu.add(menuItem);
        menuItem = new MenuItem("10");
        menuItem.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                //set maxfps to 10
                if (downloader != null){
                    downloader.setMaxFPS(10.0);
                    thisImageCanvas.fps = 10.0;
                }
            }
        });
        subMenu.add(menuItem);
        menuItem = new MenuItem("Unlimited");
        menuItem.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                //set maxfps to 30
                if (downloader != null){
                    downloader.setMaxFPS(30.0);
                    thisImageCanvas.fps = 30.0;
                }
            }
        });
        subMenu.add(menuItem);        
        popup.add(subMenu);
        /*
        MenuItem menuItem = new MenuItem("Stop");
        menuItem.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                //stop the webcam if it is already going
                if (downloader != null){
                    downloader.disconnect();
                    downloader = null;
                }
            }
        });
        popup.add(menuItem);
        */
        popup.addSeparator();
        
        //about
        menuItem = new MenuItem("About");
        menuItem.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                //display about box
                about.show();
            }
        });
        popup.add(menuItem);
        return (popup);
    }
}
